/********************************************************************
 * OBIDMS header file                                               *
 ********************************************************************/

/**
 * @file obidmscolumn.h
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 * @date 23 May 2015
 * @brief Header file for the OBIDMS functions and structures.
 */


#ifndef OBIDMS_H_
#define OBIDMS_H_


#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>
#include <string.h>
#include <stdio.h>
#include <stdbool.h>

#include "obierrno.h"
//#include "obidmscolumn.h"
//#include "obiblob_indexer.h"


#define OBIDMS_MAX_NAME (2048) 	    			/**< The maximum length of an OBIDMS name.
 	 	 	 	 	 	 	 	 	 	 	 	 */
#define INDEXER_DIR_NAME "OBIBLOB_INDEXERS"  	/**< The name of the Obiblob indexer directory.
                                	 	 	 	 */
#define TAXONOMY_DIR_NAME "TAXONOMY"    		/**< The name of the taxonomy directory.
                                	 	 	 	 */
#define MAX_NB_OPENED_COLUMNS (100) 			/**< The maximum number of columns open at the same time.
                                	 	 	 	 */
#define MAX_NB_OPENED_INDEXERS (1000) 			/**< The maximum number of indexers open at the same time.
                                	 	 	 	 */
#define MAX_PATH_LEN 4096						/**< Maximum length for the character string defining a
 	 	 	 	 	 	 	 	 	 	 	 	 *	 file or directory path.
 	 	 	 	 	 	 	 	 	 	 	 	 */


typedef int32_t obiversion_t; 	/**< TODO put dans obitypes.h
                                 */


struct OBIDMS_column;		// TODO
typedef struct OBIDMS_column* OBIDMS_column_p;

typedef struct Opened_columns_list {		// TODO Handle the problem linked to columns with the same name + means only one version
	int	 		 			nb_opened_columns;
	OBIDMS_column_p		 	columns[MAX_NB_OPENED_COLUMNS];
} Opened_columns_list_t, *Opened_columns_list_p;


struct OBIDMS_avl_group;		// TODO
typedef struct OBIDMS_avl_group* OBIDMS_avl_group_p;
typedef OBIDMS_avl_group_p Obi_indexer_p;

typedef struct Opened_indexers_list {
	int		 		 		nb_opened_indexers;
	Obi_indexer_p		    indexers[MAX_NB_OPENED_INDEXERS];
} Opened_indexers_list_t, *Opened_indexers_list_p;


/**
 * @brief A structure describing an OBIDMS instance
 *
 * A pointer to this structure is returned on creation
 * and opening of an OBITools Data Management System (DMS)
 */
typedef struct OBIDMS {
	char					directory_name[OBIDMS_MAX_NAME+1];	/**< The name of the directory
																 *   containing the DMS.
																 */
	DIR*					directory;							/**< A directory entry usable to
																 *   refer and scan the database directory.
																 */
	int     				dir_fd;								/**< The file descriptor of the directory entry
																 *   usable to refer and scan the database directory.
																 */
	DIR*					indexer_directory;						/**< A directory entry usable to
																 *   refer and scan the indexer directory.
																 */
	int						indexer_dir_fd;							/**< The file descriptor of the directory entry
																 *   usable to refer and scan the indexer directory.
																 */
	bool					little_endian;						/**< Endianness of the database.
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 */
	Opened_columns_list_p 	opened_columns;						/**< List of opened columns.
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 */
	Opened_indexers_list_p 	opened_indexers;					/**< List of opened indexers.
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 */
} OBIDMS_t, *OBIDMS_p;


/**
 * @brief Checks if an OBIDMS exists.
 *
 * @param dms_name A pointer to a C string containing the name of the database.
 *
 * @returns An integer value indicating the status of the database
 * @retval 1 if the database exists.
 * @retval 0 if the database does not exist.
 * @retval -1 if an error occurred.
 *
 * @since May 2015
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 */
int obi_dms_exists(const char* dms_name);


/**
 * @brief Creates a new OBITools Data Management instance (OBIDMS).
 *
 * A `DMS` is implemented as a directory. This function checks
 * if a directory with this name does not already exist
 * before creating the new database.
 *
 * A directory to store Obiblob indexers is also created.
 *
 * @param dms_name A pointer to a C string containing the name of the database.
 *                 The actual directory name used to store the DMS will be
 *                 `<dms_name>.obidms`.
 *
 * @returns A pointer to an OBIDMS structure describing the newly created DMS.
 * @retval NULL if an error occurred.
 *
 * @see obi_close_dms()
 * @since May 2015
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 */
OBIDMS_p obi_create_dms(const char* dms_name);


/**
 * @brief Opens an existing OBITools Data Management instance (OBIDMS).
 *
 * @param dms_name A pointer to a C string containing the name of the database.
 *
 * @returns A pointer to an OBIDMS structure describing the opened DMS.
 * @retval NULL if an error occurred.
 *
 * @see obi_close_dms()
 * @since May 2015
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 */
OBIDMS_p obi_open_dms(const char* dms_name);


/**
 * @brief Creates or opens a new OBIDMS instance.
 *
 * If the database already exists, this function opens it, otherwise it
 * creates a new database.
 *
 * @param dms_name A pointer to a C string containing the name of the database.
 *
 * @returns A pointer to an OBIDMS structure describing the OBIDMS.
 * @retval NULL if an error occurred.
 *
 * @see obi_close_dms()
 * @since May 2015
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 */
OBIDMS_p obi_dms(const char* dms_name);


/**
 * @brief Closes an opened OBITools Data Management instance (OBIDMS).
 *
 * @param dms A pointer as returned by obi_create_dms() or obi_open_dms().
 *
 * @returns An integer value indicating the success of the operation. Even on
 * 		    error, the `OBIDMS` structure is freed.
 * @retval 0 on success.
 * @retval -1 if an error occurred?-.
 *
 * @see obi_create_dms()
 * @see obi_open_dms()
 * @since May 2015
 * @author Eric Coissac (eric.coissac@metabarcoding.org)
 */
int obi_close_dms(OBIDMS_p dms);


// TODO doc
int obi_dms_is_column_name_in_list(OBIDMS_p dms, const char* column_name);


OBIDMS_column_p obi_dms_get_column_from_list(OBIDMS_p dms, const char* column_name, obiversion_t version);


void obi_dms_list_column(OBIDMS_p dms, OBIDMS_column_p column);


int obi_dms_unlist_column(OBIDMS_p dms, OBIDMS_column_p column);


Obi_indexer_p obi_dms_get_indexer_from_list(OBIDMS_p dms, const char* indexer_name);


void obi_dms_list_indexer(OBIDMS_p dms, Obi_indexer_p indexer);


int obi_dms_unlist_indexer(OBIDMS_p dms, Obi_indexer_p indexer);


char* obi_dms_get_path(OBIDMS_p dms);


/**	TODO
 * @brief Internal function getting the full path of a file or a directory from its
 *        path relative to a directory file descriptor.
 *
 * @warning The returned pointer has to be freed by the caller.
 *
 * @param directory_file_descriptor The file descriptor for the directory to which
 * 									path_name is relative.
 * @param path_name The path name for the file or directory, relative to directory_file_descriptor.
 *
 * @returns A pointer to the full path.
 * @retval NULL if an error occurred.
 *
 * @since June 2015
 * @author Celine Mercier (celine.mercier@metabarcoding.org)
 */
char* obi_dms_get_full_path(OBIDMS_p dms, const char* path_name);


/**
 * @brief Replacement function for opendirat() : open a directory relative to a directory file descriptor.
 *
 * @param directory_file_descriptor The file descriptor for the directory in which the directory should be opened.
 * @param path_name The path name for the directory to be opened, relative to directory_file_descriptor.
 *
 * @returns The file descriptor of the opened directory.
 * @retval NULL if an error occurred.
 *
 * @since June 2015
 * @author Celine Mercier (celine.mercier@metabarcoding.org)
 */
DIR* opendir_in_dms(OBIDMS_p dms, const char* path_name);


#endif /* OBIDMS_H_ */
